package Bot::Oulibot;

use strict;
use Bot::RDFDB;
use Oulibot::Util::Conf qw(get_names_by_type get_items_by_name);  
use base qw( Bot::OuliBase );
use Data::Dumper;
# this bot is just a frontend that sits on multichannels and passes back stuff from the mind

sub grammar {
    return q{ _statement : _respond | _help | _ping | _status
	      _respond : 'respond' /.+/ { return $self->respond(@item[2..$#item]) } 
	      _help : 'help' { return $self->help }
	      _status : 'status' { return $self->status }
	      _ping : 'ping' { return $self->ping }
	  };
}

sub said {
    my ($self,$said) = @_;

    # don't listen to myself; this could get loopy
    return undef if $said->{who} eq $self->name;

    if ($said->{channel} eq $self->mind) {
	# event number
	($said->{event}) = $said->{body} =~ /^(\d\d\d\d)/;
	$said->{body} =~ s/^\d\d\d\d//;
	warn($said->{body});
	# is this for me
	my $response = $self->try($said->{body});
	if ($response) {
	    ($said->{who},$said->{channel}) = $self->remember_event($said->{event});	    
	    $said->{body} = $response;
	    warn(Dumper($said));
	    return $said;
	}
    }
    else {
	# input from another channel into mind channel
        my $event = $self->new_event($said->{who},$said->{channel});
	$said->{body} = $event.' speak '.$said->{body};
	$said->{channel} = $self->mind;
	return $said;
    }
    return undef;
}

sub respond {
    my ($self,$statement) = @_;
    return $statement;
}

sub new_event {
    my ($self,$who,$channel) = @_;
    my $event = $self->event_count;
    # we store in rdf, 'cos we can
    $self->rdf->delete_match(subject => $event);
    $self->rdf->add_triple($event,'bot:request',$who);
    $self->rdf->add_triple($event,'bot:channel',$channel) if $channel;

    return $event;
}

sub event_count {
    my $self = shift;
    # starts again at 0 on restart, rolls over at 9999
    my $count = $self->{event_count} || 0;
    if ($count eq 9999) { $count = 0; }
    else { $count++; }
    $self->{event_count} = $count;
    return(sprintf("%04d",$count));
}

sub remember_event {
    my ($self,$id) = @_;
    # see Bot::RDFDB for goodness
    my ($event) = $self->rdf->triples(
				        v => ['x','y'],
				       q => [
					     [$id,'bot:request','?x'],
					     [$id,'bot:channel','?y'],
					     ]);
    
    return @{$event} if $event;
}
#
##

sub rdf {
    my $self = shift;
    return $self->{rdf} if $self->{rdf};
    my ($db) = get_names_by_type('database');
    my %db = get_items_by_name($db);
    warn(Dumper(%db));
    $self->{rdf} = Bot::RDFDB->new( database => $db{dbname},
				    table => $db{table},
				    username => $db{username},
				    password => $db{password} );
}

1;










