package Bot::RDFDB;

use strict;

use DBI;
use DBD::mysql;

use Class::MethodMaker
  new_hash_init => 'new', get_set => [ qw{ database table username password }];

{
    my $dsn; 
    sub dsn {
	my $self = shift;
	my $db = $self->database || 'oulibot';
     	$dsn = "DBI:mysql:database=".$db.";host=localhost";
	return $dsn;
    }
    my $dbh;
    
    sub dbh {
        my $self = shift;

	
	if (not $self->{_dbh}) {
	    my $u = $self->username || 'oulibot';
	    my $p = $self->password || 'bollocks';
            $self->{_dbh} = DBI->connect($self->dsn,$u,$p);
        }
        return $self->{_dbh};
    }
    sub namespace {
	my ($self,$name) = @_;
	my $out;
	if (my ($entity,$n) = $name =~ /(.*)\:(.*)/) {
	    $out = $self->entity_to_namespace($entity).$n;
	}
	else {
	    $out = 'http://frot.org/'.$self->table.'/0.1/'.$name;
	}
	return $out;
    }
    my %lookup = (
		  foaf => 'http://xmlns.com/foaf/0.1/',
		  fc => 'http://xmlns.com/foafcorp/0.1/',
		  dc => 'http://purl.org/dc/1.0/',
		  int => 'http://frot.org/interests/0.1/',
		  bot => 'http://frot.org/bot/0.1/',
		  );
		  
    sub entity_to_namespace {
	my ($self,$entity) = @_;
	return $lookup{$entity};
    }
    sub denamespace {
	my ($self,$thing) = @_;
	my ($ns,$part) = $thing =~ /(.+\/)(.+)/;
	my $entity = $self->namespace_to_entity($ns);
	return $entity.$part;
    }
}

sub triples {
    my ($self,%p) = @_;
    my @q = @{$p{q}};
    @q = map {$_->[1] = $self->namespace($_->[1]); $_} @q;
    
    my $sql = $self->triple_sql($p{v},\@q);
    my $dbh = $self->dbh;
    
    my $sth = $dbh->prepare($sql);
    eval {
        my $status = $sth->execute;
    };
    warn $@ if $@;

    my $rows = $sth->fetchall_arrayref;
    return @{$rows};
}

sub search {
    my ($self,$pred,$obj) = @_;
    my $dbh = $self->dbh;
    my $table = $self->table || 'triple';
    my $sql = "SELECT subject, object from $table where pred like '%".$pred."' AND object like '%".$obj."%'";
    my $sth = $dbh->prepare($sql);
    eval {
        my $status = $sth->execute;
    };
    warn $@ if $@;
    my $rows = $sth->fetchall_arrayref;
    return @{$rows};
}

sub add_triple {
    my ($self,$subj,$pred,$obj) = @_;
    $pred = $self->namespace($pred) if ($pred =~ m/:/);
    my $dbh = $self->dbh;
    my $table = $self->table || 'triple';
    my $sql = "INSERT INTO $table (subject,pred,object) values (".$dbh->quote($subj).",".$dbh->quote($pred).",".$dbh->quote($obj).")";

    $dbh->do($sql);
}

sub delete_triple {
    my ($self,$subj,$pred,$obj) = @_;
    my $dbh = $self->dbh;
    my $table = $self->table || 'triple';
    $dbh->do("DELETE FROM $table where subject = ".$dbh->quote($subj)." AND pred=".$dbh->quote($pred)."AND object=".$dbh->quote($obj));
}

sub delete_match {
    my ($self,%p) = @_;
    my $dbh = $self->dbh;
    my $table = $self->table || 'triple';
    if ($p{subject} and $p{pred} and (not $p{object})) {
	$dbh->do("DELETE FROM $table where subject = ".$dbh->quote($p{subject})." AND pred=".$dbh->quote($p{pred}));
    }
    if ($p{subject} and (not $p{pred}) and (not $p{object})) {
	$dbh->do("DELETE FROM $table where subject = ".$dbh->quote($p{subject}) );
    }
}


#print
#triple_sql(["x","y"],[["?x","worksFor","?y"],["?y","name","BBC"]]);
#print "\n";

sub triple_sql {
    my $self = shift;
    my ($boundvars,$clauses) = @_;
    my %vars = ();
    my @where = ();
    foreach my $v (@$boundvars) {
        $vars{$v} = [];
    }
    my $clause_idx = 1;
    foreach my $clause (@$clauses) {
        my $subject = $clause->[0];
        my $pred = $clause->[1];
        my $object = $clause->[2];

        my $subject_bound = 0;
        my $pred_bound = 0;
        my $object_bound = 0;

        foreach my $v (@$boundvars) {
            if($subject eq "?".$v) {
                push (@{$vars{$v}},"a".$clause_idx.".subject");
                $subject_bound = 1;
            }
            if($pred eq "?".$v) {
                push (@{$vars{$v}},"a".$clause_idx.".pred");
                $pred_bound = 1;
            }
            if($object eq "?".$v) {
                push (@{$vars{$v}},"a".$clause_idx.".object");
                $object_bound = 1;
            }
        }
        if(!$subject_bound) {
            push(@where,"a".$clause_idx.".subject = ".$self->dbh->quote($subject));
        }
        if(!$pred_bound) {
            push(@where,"a".$clause_idx.".pred = ".$self->dbh->quote($pred));
        }
        if(!$object_bound) {
            push(@where,"a".$clause_idx.".object = ".$self->dbh->quote($object));
        }
        $clause_idx++;
    }
    my $sql = "SELECT ";
    my @selectvars = ();
    foreach my $v (@$boundvars) {
        my $bindings = $vars{$v};
        push (@selectvars,$bindings->[0]." ".$v);
        for(my $i = 0; $i<scalar @$bindings - 1; $i++) {
            push(@where,$bindings->[$i] ." = ".$bindings->[$i+1]);
        }
    }

    $sql .= join(",",@selectvars);
    $sql .= " \nFROM ";
    my @tables = ();
    my $table = $self->table || 'triple';
    for (my $i = 1; $i < scalar(@$clauses)+1; $i++) {
        push @tables,"$table a$i";
    }

    $sql .= join(",",@tables);
    $sql .= " \nWHERE ";
    $sql .= join(" \nAND ",@where);
    return $sql;
}

sub uri {
    my ($self,$thing) = @_;
    my $namespace = $self->namespace;
    return $namespace . $thing;
}

sub random_id {
   my @things = (0..9);
   return '_:id'.join '', (map { @things[rand @things] } 0..6);
}

1;







