package Oulibot;

use strict;

use Oulibot::DB;
use Oulibot::Grabber;
use Oulibot::Translator;

use HTML::Entities;

use Class::MethodMaker new_hash_init => 'new', get_set => [ qw{ table context filter source matched}];

sub db {
    my $self = shift;
    return $self->{_db} if $self->{_db};
    $self->{_db} = Oulibot::DB->new( database => 'oulibot', username => 'oulibot' );
}

sub translator {
    my $self = shift;
    return $self->{_tr} if $self->{_tr};
    $self->{_tr} = Oulibot::Translator->new;
}
 
sub random_seed {
    my $self = shift;
    my $seed = $self->seed;
    return $seed->[rand @{$seed}];
}

## 

sub contextualise {
    my ($self,$message) = @_;

    $self->db->remember_input($message) if $message;

    $message ||= $self->random_past_input;
    my $context = $self->context || 4;

    my @pages = $self->db->search(searchterm => $message,
				  context => $context,
				  source => $self->source);

    warn(scalar(@pages) . " matching things\n"); 
    $self->matched(scalar(@pages));
    my @words = split ' ',$message;
    @words = $self->db->drop_common_words(@words);

    my @translated;
    foreach my $word (@words) {
        push @translated, $self->vague_translation($word);
    }

    foreach my $t (@translated) {
	push @pages, $self->db->search($t,2,$self->table);
	last if scalar(@pages) > 9;
    }

    my $limit = $self->pronouncements || 42;
    my @responses = $self->make_responses(\@pages,$limit);

    return @responses;
}

sub make_responses {
    my ($self,$pages,$responses) = @_;

    my @text;
    foreach (@{$pages}) {
        push @text, split("\n",$_);
    }

    my $filterpkg = ucfirst($self->filter) || 'Base';
    $filterpkg = 'Oulibot::Filter::'.$filterpkg;
    eval "require $filterpkg";
    my $filter = $filterpkg->new;

    my @potential_text = $filter->filter(@text);

    my @return;
    if ($responses) {
	if (scalar(@potential_text) < $responses) {
	    push @return, @potential_text;
	}
	else {
	    for (1..$responses) {
		push @return, $potential_text[rand @potential_text];
	    }
	}
    }
    else {
	@return = @potential_text;
    }
    @return = map { HTML::Entities::decode($_) } @return;
    return @return;
}

sub aphorism {
    my ($self, $question) = @_;
    my @text = $self->contextualise($question);
    return $text[rand @text];
}

sub learn_page {
    my ($self,$uri) = @_;
    # ideally send message to grabbot or look at uri
    my $grabber = Oulibot::Grabber->new(source => $self->source);
    $grabber->grab_without_checking($uri);
}

sub is_interesting {
    my ($self, @text) = @_;

    # cheese heuristics
    my ($total, $long, $short, $numeric) = (0,0,0,0);
    foreach (@text) {
        my $line = $_;
        chomp $line;

        if ($line =~ m/\w/) {
            $total++;
            $numeric++ if ($line =~ m/\d\d\d/);
            if (length($line) > 100) { $long++; } else { $short++;  }
        }
    }
    warn("t $total l $long s $short n $numeric\n");
    return 0 if $total < 23;
    return 0 if (($numeric*3) > $total);
    return 0 if (($long*23) < $short);
#    return 0 if (($long*4) < $total);
    return 1;
}

sub vague_translation {
    my ($self, $word) = @_;
    my $tr = $self->translator;
    return $tr->translate($word);
}

sub random_past_input {
    my $self = shift;
    my $random = $self->db->random_input;
    return $random;
}

sub pronouncements {
    my ($self, $p) = @_;
    if (not $self->{p}) {
        $self->{p} = $p || 42;
    }
    return $self->{p};
}

sub min_line_length {
    my ($self, $line) = @_;
    if (not $self->{min_line_length}) {
        $self->{min_line_length} = $line || 3;
    }
    return $self->{min_line_length};
}

sub seed {
    my ($self, $seed) = @_;
    if (not $self->{seed}) {
        if (ref $seed eq 'ARRAY') {
            $self->{seed} = $seed;
        }
        elsif ($seed) {
            $self->{seed} = [$seed];
        }
        else {
            $self->{seed} = ['do not leave it is not real'];
        }
    }
    return $self->{seed};
}

# fisher yates shuffle
{
    sub shuffle {
        my ($self,@array) = @_;
        for (my $i = scalar(@array); --$i; ) {
            my($j) = int(rand($i + 1));
            next() if ($i == $j);
            @array[$i, $j] = @array[$j, $i];
        }
    }
}

##

1;














