package Oulibot::Babel;

use strict;
use WWW::Babelfish;
use Oulibot::DB;

use Class::MethodMaker
  new_hash_init => 'new', get_set => [ qw{  }];

{
    my $babelfish;

    sub babelfish {
        my $self = shift;
        return $self->{babelfish} if $self->{babelfish};
        eval {
            $babelfish = new WWW::Babelfish( 'agent' => 'Mozilla/8.0' );
        };
        warn($@) if $@;
        $self->{babelfish} = $babelfish;
        return $self->{babelfish};
    }

    my @languages;
    sub languages {
        my $self = shift;
        return $self->{languages} if $self->{languages};
        $babelfish = $self->babelfish;
        return if not $babelfish;
        @languages = $babelfish->languages;
	@languages = grep{!/[Korean|Japanese]/} @languages; 
        $self->{languages} = \@languages;
    }

    sub db {
        my $self = shift;
        if (not $self->{_db}) {
            $self->{_db} = Oulibot::DB->new;
        }
        return $self->{_db};
    }

    my %lookup = (
		  'UK' => 'English',
		  'ES' => 'Spanish',
		  'DE' => 'German',
		  'FR' => 'French',
		  'JP' => 'Japanese',
		  'KR' => 'Korean',
		  'IT' => 'Italian',
		  'PT' => 'Portuguese'
		  );
    
    sub lookup {
	my ($self,$code) = @_;
	return \%lookup if not $code;
	return $lookup{$code};
    }
}

sub translate_random {
    my ($self,$said,$source) = @_;
    if ($source) {
	if ($source =~ m/[A-Z][A-Z]/) {
	    $source = $self->lookup($source);
	}
    }
    
    # cultural bias
    $source ||= 'English';
    my $babelfish = $self->babelfish;
    return undef if not $babelfish;

    my @languages = @{$self->languages};

    my $out_lang = $languages[rand @languages];
    my $translated = $babelfish->translate( 'source' => 'English',
                                            'destination' => $out_lang,
                                            'text' => $said,
                                            'delimiter' => "\n\t",
                                          );
    if ($translated) {
        if (($translated ne $said) && ($translated ne '&nbsp;')) {
            $self->db->add_translation($said,$out_lang,$translated);
	    return $translated;
	}
    }
    return undef;
}

sub vague_translation {
    my ($self,$said) = @_;
    my @translations = $self->db->translate($said);

    if (scalar(@translations) eq 0) {
        my @words = split(' ',$said);
        return undef if scalar(@words) eq 1;
        my @translated;
        foreach (@words) {
            my @translations = $self->db->translate($_);
            push @translated, @translations;
            return join(' ',@translated) if scalar(@translated) > 4;
        }
        return join(' ',@translated);
    }
    else {
        return join(' ',@translations);
    }
}

1;











