package Oulibot::Client::IRC;

# adapted by zool from 
# Rocco's IRC bot stuff.

use strict;

use POE::Session;
use POE::Component::IRC;

sub MSG_SPOKEN    () { 0x01 }
sub MSG_WHISPERED () { 0x02 }
sub MSG_EMOTED    () { 0x04 }

use Oulibot::Util::Conf;
#use Util::Link;
use Oulibot::Server::Web;

#------------------------------------------------------------------------------
# Spawn the IRC session.

my ($service) = get_names_by_type('irc');

my %conf = get_items_by_name($service);

my @bots = @{$conf{bot}};

foreach my $bot (@bots) {
    sleep(1);
    my $bot_object;
    my $class = 'Bot::'.ucfirst($bot);
    eval "require $class";
    if ($@) {
	warn "no $class bot!";
	next;
    }
    else {
	$bot_object = $class->new;
    }
  POE::Component::IRC->new($bot);

  POE::Session->create
    ( inline_states =>
      { _start => sub {
          my ($kernel, $heap, $session) = @_[KERNEL, HEAP, SESSION];

          $kernel->alias_set( "client_$bot" );
          $kernel->post( $bot => register => 'all' );

          $heap->{server_index} = 0;

          # Keep-alive timer.
          $kernel->delay( autoping => 60 );

          $kernel->yield( 'connect' );
      },

      # ctcp ping ourselves every minute to generate network traffic so
      # we can detect a broken connection faster
      autoping => sub {
          my $kernel = $_[KERNEL];
          $kernel->post( $bot => ctcp => $bot => "PING 123456789" );
          $kernel->delay( autoping => 60 );
      },

      connect => sub {
          my ($kernel, $heap) = @_[KERNEL, HEAP];

          $kernel->post( $bot => connect =>
                         { Debug    => 0,
                           Nick     => $bot,
                           Server   => $conf{server},
                           Port     => 6667,
                           Username => $bot,
                           Ircname  => $conf{iname}.' '.$bot,
                         }
                       );

#          $heap->{server_index}++;
#          $heap->{server_index} = 0
#            if $heap->{server_index} >= @{$conf{server}};
      },

      join => sub {
        my ($kernel, $channel) = @_[KERNEL, ARG0];
        $kernel->post( $bot => join => $channel );
      },

      _stop => sub {
        my $kernel = $_[KERNEL];
        $kernel->post( $bot => quit => $conf{quit} );
      },

      # events to be totally ignored
      map({$_ => sub {} } qw(irc_002 irc_003 irc_004 irc_005 irc_250 irc_251
			     irc_254 irc_255 irc_265 irc_266 irc_301 irc_306
			     irc_353 irc_366 irc_372 irc_375 irc_376
			     irc_ctcp_ping irc_mode irc_join irc_quit)),

      _default => sub {
        my ($state, $event, $args) = @_[STATE, ARG0, ARG1];
        $args ||= [ ];
        #log_event("default $state = $event @$args)");
        return 0;
      },

      irc_001 => sub {
        my ($kernel, $heap) = @_[KERNEL, HEAP];

        if (defined $conf{flags}) {
          $kernel->post( $bot => mode => $bot => $conf{flags} );
        }
        $kernel->post( $bot => away => $conf{away} );

	if ($bot eq $conf{nick}) {
	    my @channels = @{$conf{channel}};
	    push @channels, $conf{mind};
	    foreach my $channel (@channels) {
		$kernel->yield( join => "\#$channel" );
	    }
	}
	else {
	    my $channel = $conf{mind};
	    $kernel->yield( join => "\#$channel" );
	}
        $heap->{server_index} = 0;
      },

      irc_ctcp_version => sub {
        my ($kernel, $sender) = @_[KERNEL, ARG0];
        my $who = (split /!/, $sender)[0];
        log_event("ctcp version from $who");
        $kernel->post( $bot => ctcpreply => $who, "VERSION $conf{cver}" );
      },

      irc_ctcp_clientinfo => sub {
        my ($kernel, $sender) = @_[KERNEL, ARG0];
        my $who = (split /!/, $sender)[0];
        log_event("ctcp clientinfo from $who");
        $kernel->post( $bot => ctcpreply =>
                       $who, "CLIENTINFO $conf{ccinfo}"
                     );
      },

      irc_ctcp_userinfo => sub {
        my ($kernel, $sender) = @_[KERNEL, ARG0];
        my $who = (split /!/, $sender)[0];
        log_event("ctcp userinfo from $who");
        $kernel->post( $bot => ctcpreply =>
                       $who, "USERINFO $conf{cuinfo}"
                     );
      },

      irc_invite => sub {
        my ($kernel, $who, $where) = @_[KERNEL, ARG0, ARG1];
        $kernel->yield( join => $where );
      },

      irc_kick => sub {
        my ($kernel, $who, $where, $isitme, $reason) = @_[KERNEL, ARG0..ARG4];
        log_event("$who was kicked from $where: $reason");
        # $kernel->delay( join => 15 => $where );
      },

      irc_disconnected => sub {
        my ($kernel, $bot) = @_[KERNEL, ARG0];
        log_event("Lost connection to server $bot.");
        $kernel->delay( connect => 60 );
      },

      irc_error => sub {
        my ($kernel, $error) = @_[KERNEL, ARG0];
        log_event("Server error occurred: $error");
        $kernel->delay( connect => 60 );
      },

      irc_socketerr => sub {
        my ($kernel, $error) = @_[KERNEL, ARG0];
        log_event("IRC client ($bot): socket error occurred: $error");
        $kernel->delay( connect => 60 );
      },

      irc_public => sub {
        my ($kernel, $who, $where, $msg) = @_[KERNEL, ARG0..ARG2];
        $who = (split /!/, $who)[0];
        $where = $where->[0];
#	name, eval, method, response
	my $response = $bot_object->said({who => $who,
					 body => $msg,
					 channel => $where});

	if ($response && $response->{body}) {
	    my $who = ($response->{channel} eq "msg") ? $response->{who} : $response->{channel};
	    my @say;
	    if ($response->{body} =~ m/\n/) {
		@say = split("\n",$response->{body});
	    }
	    else {
		@say = ($response->{body});
	    }
	    foreach (@say) {
		$kernel->post($bot, 'privmsg',$who,$_);
	    }
	}
#        log_event("<$who:$where> $msg");

      },

      irc_msg => sub {
        my ($kernel, $who, $msg) = @_[KERNEL, ARG0, ARG2];

        $who = (split /!/, $who)[0];
      },
    },
  );
}

sub log_event {
    my $text = shift;
    # print for now, maybe syslog later
    print $text, "\n";
}

#------------------------------------------------------------------------------
1;


