package Oulibot::DB;

use strict;

use base qw(Bot::RDFDB);

##

sub remember_page {
    my ($self,%p) = @_;

    my ($seen) = $self->triples( v => ['x'],
				 q => [[$p{uri},'dc:content','?x']] );
    if (not $seen) {
	my $id = $self->add_page($p{content});
	$self->add_triple($p{uri},'dc:content','_id:'.$id);
	my $time = localtime;
	$self->add_triple($p{uri},'dc:date',$time);
	$self->add_triple($p{uri},'dc:title',$p{title}) if $p{title};
    }
}

sub remember_input {
    my $self = shift;
    my $input = shift;
    my $id = $self->random_id;
    $self->add_triple($id,'bot:question',$input);
    my $time = localtime;
    $self->add_triple($id,'dc:date',$time);
}

sub remember_uri {
    my ($self,$uri) = @_;
    my ($found) = $self->triples( v => ['x'],
				  q => [[$uri,'dc:date','?x']]);
    
    if (not $found) {
	my $time = localtime;
	$self->add_triple($uri,'dc:date',$time);
    }
}

sub add_page {
    my ($self,$content) = @_; 
    $self->dbh->do("INSERT INTO pages VALUES (NULL,".$self->dbh->quote($content).")");
    my $id = $self->dbh->{'mysql_insertid'};
    return $id;
}

sub add_translation {
    my ($self,$subj,$pred,$obj) = @_;
    my $dbh = $self->dbh;
    $dbh->do("INSERT INTO words values (".$dbh->quote($subj).",".$dbh->quote($pred).",".$dbh->quote($obj).")");

}

sub translate {
    my ($self,$text) = shift;
    my $dbh = $self->dbh;
    my $sth = $dbh->prepare("SELECT pred, object FROM words where subject like '%$text%'");
    $sth->execute;
    my @return;
    while (my $row = $sth->fetchrow_hashref) {
        push @return, $row->{object};
    }
    return @return;
}

sub random_input {
    my $self = shift;
    my @questions = $self->triples(v => ['x','y'],
				   q => [['?x','bot:question','?y']]);
    @questions = map {$_->[1]} @questions;

    return $questions[rand @questions];
}

sub random_page {
    my $self = shift;
    my $sth = $self->dbh->prepare("SELECT content from pages order by rand() limit 1");
    $sth->execute;
    while (my $row = $sth->fetchrow_hashref) {
        return $row->{content};
    }
}

sub last_page {
    my $self = shift;
    my $dbh = $self->dbh;
    my $sth = $dbh->prepare("SELECT content from pages limit 1");
    $sth->execute;
    while (my $row = $sth->fetchrow_hashref) {
        return $row->{content};
    }
}

sub search {
    my ($self,%p) = @_;
    my $context = $p{context} || 6;

    my $sth = $self->dbh->prepare("SELECT id, content from pages WHERE MATCH (content) AGAINST (".$self->dbh->quote($p{searchterm}).") LIMIT $p{context}");
    $sth->execute;

    my @return;
    while (my $page = $sth->fetchrow_hashref) {
	if ($p{source}) {
	    my ($s) = $self->triples(v => ['x','y'],
				   q => [['?x','dc:content','_id:'.$page->{id}],
					 ['?x','bot:source','?y']]);

	    push @return, $page->{content} if ($s->[1] eq $p{source}); 
	}
	else {
	    push @return, $page->{content};
	}
    }
    return @return;
}

sub seen_page {
    my ($self,$uri) = @_;
    my ($seen) = $self->triples( v => ['x'],
				 q => [[$uri,'dc:date','?x']]);
    return $seen->[0] if $seen;
    return undef;
}

sub count {
    my ($self, $table, $subscript) = @_;

    my $query = "SELECT count(*) from $table";
    $query .= ' '.$subscript if $subscript;

    my $dbh = $self->dbh;
    my $sth = $dbh->prepare($query);
    $sth->execute;
    my $total;
    while (my $row = $sth->fetchrow_arrayref) {
        $total = $row->[0];
    }
    return $total;
}

sub drop_common_words {
    my ($self, @words) = @_;
    my @common = $self->overcommon;
    foreach my $c (@common) {
	@words = grep {!/$c/} @words; 
    }
    return @words;
}

sub overcommon {
    return ('and','is','the','this','that','then','who','but','in','le','la','el','este','ce');
}


1;


















