package Oulibot::Filter::Base;

use strict;

use Class::MethodMaker
  new_hash_init => 'new', get_set => [ qw{ min_words }];

sub filter {
    my ($self,@text) = @_;
    my @out;

    foreach (@text) {

        my $line = $_;
        # we get a lot of blank lines for some reason
        next if $line !~ m/\w/;
        # lots of digits look boring
        next if $line =~ m/(\d\d\d)/;
	# catches many codelike statements
	next if $line =~ m/\{/;

        $line =~ s/\t/ /g;
        $line =~ s/<\\?i>//g;
        $line =~ s/<\\?b>//g;
        $line =~ s/\r\n/ /g;
        $line =~ s/\n/ /g;
        $line =~ s/( +)/ /g;
        $line =~ s/(^\s+)//;

        my @words = split(' ',$line);

        my $min_words = $self->min_words || 5;
        # four or less words at once look boring
        next if scalar(@words) < $min_words;

        if ($line =~ m/\./) {
            my @chunks = split('.',$line);
            foreach my $chunk (@chunks) {
                if (length($chunk) > 10) {
                    push @out, $chunk;
                }
                else {
                    my $next = shift @chunks;
                    if (not $next) {
                        push @out, $chunk;
                    }
                    else {
                        unshift @chunks, $chunk.$next;
                    }
                }
            }
        }
        else {
            push @out, $line;
        }
    }
    return @out;
}

1;
