package Oulibot::Grabber;

use strict;

use Oulibot;
use Oulibot::DB;
use LWP::UserAgent;
use HTML::Parser;
use Class::MethodMaker
  new_hash_init => 'new', get_set => [ qw{ source site }];

sub check_uri {
    my ($self, $uri) = @_;

    return undef if $uri =~ m/\.pdf$/;
    return undef if $uri =~ m/\.mp3$/;

    if ($uri !~ m/^http/) {
        $uri = 'http://'.$self->site.$uri;
    }
    my $seen = $self->db->seen_page($uri);
    return undef if $seen;

    return $uri;
}

sub get_page {
    my ($self,$uri) = @_;
    my $ua = LWP::UserAgent->new(timeout => 30);
    my $response = $ua->get($uri);
    my $page = $response->content;
    return $page;
}

sub grab_page_as_xhtml {
    my ($self,$uri) = @_;
    $uri = "http://cgi.w3.org/cgi-bin/tidy?docAddr=$uri";
    return $self->grab($uri);
}

sub grab {
    my ($self,$request) = @_;
    my $uri = $self->check_uri($request);
    return if not $uri;

    $self->db->remember_uri($uri);
    my $page = $self->get_page($uri);
    
    return if not $page;
    
    my %parsed = $self->parse_page($page);
    my $interesting = $self->think_about(@{$parsed{text}});
    my $content = join("\n",@{$parsed{text}});

    if ($interesting) {
        $self->db->remember_page(uri => $uri,
				 content => $content,
				 title => $parsed{title},
				 );
	warn("interesting -> $uri");
        return $uri;
    }
    else {
        warn("dull -> $uri");
        return undef;
    }
}

sub parse_page {
    my ($self,$page) = @_;
    my ($title, @text);

    my $p = HTML::TokeParser->new(\$page);

    if ($p->get_tag("title")) {
	$title = $p->get_trimmed_text;
    }

    while (my $token = $p->get_token) {
	my $text = $p->get_text;
	push @text, $text if ($text =~ m/\w/);
    }

    return ( title => $title,
	     text => \@text );
}

sub think_about {
    my ($self, @things) = @_;
    return 1 if Oulibot::is_interesting(@things);
    return undef;
}

sub grab_without_checking {
    my ($self,$uri) = @_;
    $self->db->remember_uri($uri);

    my $ua = LWP::UserAgent->new(timeout => 20);
    my $response = $ua->get($uri);
    my $page = $response->content;

    my %parsed = $self->parse_page($page);
    my $content = join(' ',@{$parsed{text}});

    $self->db->remember_page(uri => $uri,
			     content => $content,
			     title => $parsed{title});
}

sub grab_as_xhtml {
    my ($self, $request) = @_;
    my $uri = $self->check_uri($request);
    return if not $uri;
    $uri = "http://cgi.w3.org/cgi-bin/tidy?docAddr=$uri";
    $self->grab($uri);
}

sub db {
    my $self = shift;
    if (not $self->{_db}) {
        $self->{_db} = Oulibot::DB->new;
    }
    return $self->{_db};
}



1;


















