package Oulibot::Parsers::Base;

use strict;

sub new {
    my $class = shift;
    my $self = bless { }, ref $class || $class;
    return $self;
}

sub parse {
    #    my $self = shift
    die "override!";
}

=head1 Oulibot::Parsers::Base

=cut

=head2 SUBCLASSING

=cut

=item METHOD parse_search

this is the method your parser subclass should override.
simply, called on an object of this class it accepts a string which is a page of text; a page of search results from $search_engine_of_your_choice

it returns a data structure which is an array reference of hash references to uris in this structure

      $item = { title => $title,
                uri => $uri,
                desc => undef };

to easily subclass this, put your package with the overridden method in the Oulibot::Parsers:: namespace and call it this wise:

      my $search = Oulibot::Searcher->new(
                                    site => 'www.google.com',
                                    search_uri => '/search?q=SEARCHTERM',
                                    parser => 'Google'
                                   );

for example. i know this is very crufty and as i mentioned in the README would like a bot-bot interface to do this but it is all right for quick experiments.


=cut



1;

