package Oulibot::Searcher;

use strict;
use Class::MethodMaker
  new_hash_init => 'new', get_set => [ qw{ engine }];
use Oulibot::Grabber;
use Data::Dumper;

use LWP::UserAgent;

{

    sub db {
        my $self = shift;
        if (not $self->{_db}) {
            $self->{_db} = Oulibot::DB->new;
        }
        return $self->{_db};
    }

    sub site {
        my ($self,$site) = @_;
        return $self->{site} if $self->{site};
        if (not $site && not $self->{site}) {
            die "i need a site url to search! give me a site => www.foo.com input \n\n";
        }
        if (not $self->{site}) {
            $self->{site} = $site;
        }
        return $self->{site};
    }

    ##

    sub search_uri {
        my ($self, $uri) = @_;
        return $self->{search_uri} if $self->{search_uri};
        if (not $uri && not $self->{search_uri}) {
            die "i need a search uri, in the cheesy format '/search?q=SEARCHTERM' where SEARCHTERM is the string that gets interpolated with the input string. hacky, eh? \n\n";
        }
        if (not $self->{search_uri}) {
            $self->{search_uri} = $uri;
        }
        return $self->{search_uri};
    }
    sub parser {
        my ($self, $parser) = @_;
        return $self->{parser} if $self->{parser};
        if (not $parser && not $self->{parser}) {
            die "i need a name for the parser module in the Oulibot::Parser hierarchy, e.g. 'Google'";

        }
        if (not $self->{parser}) {
            $self->{parser} = $parser;
        }
        return $self->{parser};
    }

    my @engines = ('AllTheWeb','Google');
    sub defaultengines {
        return \@engines;
    }
}

sub search {
    my ($self,$text) = @_;
    my $uris = $self->search_for_pages($text);
    return $uris;
}

sub search_for_uris {
    my ($self,$request) = @_;
    my $uris = $self->do_search($request);
    return $uris;
}

sub search_for_new_uris {
    my ($self,$request) = @_;
    my $uris = $self->do_search($request);
    my @out;
    foreach (@$uris) {
        push @out, $_ if not $self->db->seen_page($_);
    }
    return \@out;
}

sub search_for_pages {
    my ($self,$request) = @_;
    my $uris = $self->do_search($request);
    warn(Dumper($uris));
    my $site = $self->site;
    my @return;
    my $grabber = Oulibot::Grabber->new( site => $self->site );

    foreach (@{$uris}) {
        my $uri = $_;

        if ($uri !~ m/^http:/) {
            $uri = 'http://'.$site.$uri;
        }
        #warn("grabbing");
        push @return, $grabber->grab($uri);
    }
    return \@return;
}

sub do_search {
    my $self = shift;

    my $search_for = shift;
    my $return;
    $search_for =~ s/ /%20/g;

    my $site = $self->site or die "what site?";
    my $search_uri = $self->search_uri;
    $search_uri =~ s/SEARCHTERM/$search_for/;
    #warn($search_uri);
    my $ua = LWP::UserAgent->new(timeout => 30);
    $ua->agent("oulibot/1.0 http://www.oulibot.org/about.html");
    my $response;
    eval {
        #warn("http://".$site.$search_uri);
        $response = $ua->get("http://".$site.$search_uri);
    };
    warn($@) if $@;
    my $page = $response->content;

    my $class = $self->parser($site);
    $class = 'Oulibot::Parsers::'.$class;
    eval "require $class";

    my $parser = $class->new;
    my $results = $class->parse_search($page);
    my @uris = map {$_->{uri}} @{$results};
    return \@uris;

}


1;







