package Oulibot::Translator;

use strict;
use base qw(Word::DB);
use Oulibot::Babel;
use Oulibot::Util::Conf;
 
sub babel {
    my $self = shift;
    return $self->{babel} if $self->{babel};
    $self->{babel} = Oulibot::Babel->new;
}

sub database {
    my %conf = get_items_by_name('database');
    return $conf{dbname};
}

sub translate {
    my ($self,$word,$lang) = @_;
    # cultural bias
    $lang ||= 'en/uk';
    my $word = $self->exists(word => $word,
			     lang => $lang);
    if (not $word) {
	$word = $self->add(word => $word,
			   lang => $lang);
    }
    
    my @means = $self->rdf->triples(
				    v => ['x','y'],
				    q => [[$word,'word:means','?x'],
						['?x','dc:lang','?y']
						]
				    );
    my @words;
    foreach (@means) {
	next if $_->[1] eq $lang;
	my $word = $_->[0];
	$word = $self->rdf->denamespace($word);
	$word =~ s/_/ /g;
	push @words, $word;
    }
}

sub learn_translations {
    my ($self,$word,$lang) = @_;
    $lang ||= 'UK';
    my @translated;
    for (0..2) {
	push @translated, $self->babel->translate_random($word,$lang);
    }
    return @translated;
}

sub rough_translate {
    my ($self,%p) = @_;
    my $statement = $p{statement};
    my $lang = $p{lang} || 'English';

    my @words = split(' ',$statement);
    @words = map {s/\W//g} @words;
    
    my %return;
    foreach my $w (@words) {
	my @trans = $self->translate($w);
	if (not @trans) {
	    @trans = $self->learn_translations($w);
	}
	$return{$w} = \@trans;
    }
    my $out;
    while (my ($key,$value) = each %return) {
	my @out = @{$value};
	$out .= $out[rand @out].' '; 
    }
    return $out;
}

sub exists {
    my ($self,%p) = @_; 
    my $rdf = $self->rdf;
    my $word = ucfirst($p{word});
    $word =~ s/ /_/g;
    my $lang = $p{lang};
    my $code = $self->babel->lookup($lang);
    my (@exists) = $self->rdf->triples( v => ['x'],
					q => [[$rdf->namespace("$code:$word"),$rdf->namespace('dc:lang'),'?x']]
					);
    foreach (@exists) {
	return $word if $_->[0] eq $lang;
    }
    return undef;
}


sub add {
    my ($self,$word) = @_;
}


1;
